@@ -59,9 +59,9 @@ audio_output audio_alsa = {
     .flush = &flush,
     .delay = &delay,
     .play = &play,
-    .mute = NULL,      // to be set later on...
-    .volume = NULL,    // to be set later on...
-    .parameters = NULL // to be set later on...
+    .mute = &mute,
+    .volume = &volume,
+    .parameters = &parameters
 };
 
 static pthread_mutex_t alsa_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -70,7 +70,8 @@ static unsigned int desired_sample_rate;
 
 static snd_pcm_t *alsa_handle = NULL;
 static snd_pcm_hw_params_t *alsa_params = NULL;
-
+static snd_ctl_t *ctl = NULL;
+static snd_ctl_elem_id_t *elem_id = NULL;
 static snd_mixer_t *alsa_mix_handle = NULL;
 static snd_mixer_elem_t *alsa_mix_elem = NULL;
 static snd_mixer_selem_id_t *alsa_mix_sid = NULL;
@@ -82,7 +83,7 @@ static char *alsa_mix_dev = NULL;
 static char *alsa_mix_ctrl = "Master";
 static int alsa_mix_index = 0;
 static int hardware_mixer = 0;
-
+static int has_softvol = 0;
 
 static int play_number;
 static int64_t accumulated_delay, accumulated_da_delay;
@@ -289,18 +290,39 @@ static int init(int argc, char **argv) {
         audio_alsa.parameters = &parameters; // likewise the parameters stuff
         if (alsa_mix_mindb == SND_CTL_TLV_DB_GAIN_MUTE) {
           // Raspberry Pi does this
-          debug(1, "Lowest dB value is a mute.");
+          debug(1, "Lowest dB value is a mute -- try minimum volume +1");
           if (snd_mixer_selem_ask_playback_vol_dB(
-                  alsa_mix_elem, alsa_mix_minv + 1, &alsa_mix_mindb) == 0)
-            debug(1, "Can't get dB value corresponding to a \"volume\" of 1.");
+                  alsa_mix_elem, alsa_mix_minv + 1, &alsa_mix_mindb) != 0)
+            debug(1, "Can't get dB value corresponding to a minimum volume + 1.");
         }
         debug(1, "Hardware mixer has dB volume from %f to %f.",
               (1.0 * alsa_mix_mindb) / 100.0, (1.0 * alsa_mix_maxdb) / 100.0);
       } else {
         // use the linear scale and do the db conversion ourselves
         debug(1, "note: the hardware mixer specified -- \"%s\" -- does not have "
-                 "a dB volume scale, so it can't be used.",
+                 "a dB volume scale, so it can't be used. Trying software "
+                 "volume control.",
               alsa_mix_ctrl);
+
+        if (snd_ctl_open(&ctl, alsa_mix_dev, 0) < 0)
+          die("Cannot open control \"%s\"", alsa_mix_dev);
+        if (snd_ctl_elem_id_malloc(&elem_id) < 0)
+          die("Cannot allocate memory for control \"%s\"", alsa_mix_dev);
+        snd_ctl_elem_id_set_interface(elem_id, SND_CTL_ELEM_IFACE_MIXER);
+        snd_ctl_elem_id_set_name(elem_id, alsa_mix_ctrl);
+
+        if (snd_ctl_get_dB_range(ctl, elem_id, &alsa_mix_mindb,
+                                               &alsa_mix_maxdb) == 0) {
+          debug(1, "Volume control \"%s\" has dB volume from %f to %f.",
+                    alsa_mix_ctrl,
+                    (1.0 * alsa_mix_mindb) / 100.0,
+                    (1.0 * alsa_mix_maxdb) / 100.0);
+          has_softvol = 1;
+        } else {
+          debug(1, "Cannot get the dB range from the volume control \"%s\"",
+                    alsa_mix_ctrl);
+        }
+
         /*
         debug(1, "Min and max volumes are %d and
         %d.",alsa_mix_minv,alsa_mix_maxv);
@@ -465,7 +487,7 @@ int open_alsa_device(void) {
   
   if (alsa_characteristics_already_listed==0) {
   		alsa_characteristics_already_listed=1;
-  		
+  		int log_level = 2; // the level at which debug information should be output
   		int rc;
   		snd_pcm_access_t access_type;
   		snd_pcm_format_t format_type;
@@ -476,7 +498,7 @@ int open_alsa_device(void) {
 			int dir;
 			snd_pcm_uframes_t frames;
 
-			debug(1,"PCM handle name = '%s'",
+			debug(log_level,"PCM handle name = '%s'",
 						 snd_pcm_name(alsa_handle));
 			
 //			ret = snd_pcm_hw_params_any(alsa_handle, alsa_params);
@@ -486,103 +508,103 @@ int open_alsa_device(void) {
 //						alsa_out_dev);
 //			}
 
-			debug(1,"alsa device parameters:");
+			debug(log_level,"alsa device parameters:");
 
 			snd_pcm_hw_params_get_access(alsa_params,&access_type);			
-			debug(1,"  access type = %s", snd_pcm_access_name(access_type));
+			debug(log_level,"  access type = %s", snd_pcm_access_name(access_type));
 
 			snd_pcm_hw_params_get_format(alsa_params,&format_type);
-			debug(1,"  format = '%s' (%s)",snd_pcm_format_name(format_type),snd_pcm_format_description(format_type));
+			debug(log_level,"  format = '%s' (%s)",snd_pcm_format_name(format_type),snd_pcm_format_description(format_type));
 
 			snd_pcm_hw_params_get_subformat(alsa_params,&subformat_type);
-			debug(1,"  subformat = '%s' (%s)",snd_pcm_subformat_name(subformat_type),snd_pcm_subformat_description(subformat_type));
+			debug(log_level,"  subformat = '%s' (%s)",snd_pcm_subformat_name(subformat_type),snd_pcm_subformat_description(subformat_type));
 
 			snd_pcm_hw_params_get_channels(alsa_params, &uval);
-			debug(1,"  number of channels = %u", uval);
+			debug(log_level,"  number of channels = %u", uval);
 
 			sval = snd_pcm_hw_params_get_sbits(alsa_params);
-			debug(1,"  number of significant bits = %d", sval);
+			debug(log_level,"  number of significant bits = %d", sval);
 			
 			snd_pcm_hw_params_get_rate(alsa_params, &uval, &dir);
  			switch (dir) {
 			  case -1:
-			    debug(1,"  rate = %u frames per second (<).", uval);
+			    debug(log_level,"  rate = %u frames per second (<).", uval);
 			    break;
 			  case 0:
-			    debug(1,"  rate = %u frames per second (precisely).", uval);
+			    debug(log_level,"  rate = %u frames per second (precisely).", uval);
 			    break;
 			  case 1:
-			    debug(1,"  rate = %u frames per second (>).", uval);
+			    debug(log_level,"  rate = %u frames per second (>).", uval);
 			    break;
 			}
 
 			if (snd_pcm_hw_params_get_rate_numden(alsa_params,&uval, &uval2)==0)
-				debug(1,"  precise (rational) rate = %.3f frames per second (i.e. %u/%u).", uval, uval2, ((double)uval)/uval2);
+				debug(log_level,"  precise (rational) rate = %.3f frames per second (i.e. %u/%u).", uval, uval2, ((double)uval)/uval2);
 			else
-				debug(1,"  precise (rational) rate information unavailable.");
+				debug(log_level,"  precise (rational) rate information unavailable.");
 
 			snd_pcm_hw_params_get_period_time(alsa_params,&uval, &dir);			
 			switch (dir) {
 			  case -1:
-			    debug(1,"  period_time = %u us (<).", uval);
+			    debug(log_level,"  period_time = %u us (<).", uval);
 			    break;
 			  case 0:
-			    debug(1,"  period_time = %u us (precisely).", uval);
+			    debug(log_level,"  period_time = %u us (precisely).", uval);
 			    break;
 			  case 1:
-			    debug(1,"  period_time = %u us (>).", uval);
+			    debug(log_level,"  period_time = %u us (>).", uval);
 			    break;
 			}
 
 			snd_pcm_hw_params_get_period_size(alsa_params,&frames, &dir);
 			switch (dir) {
 			  case -1:
-			    debug(1,"  period_size = %lu frames (<).", frames);
+			    debug(log_level,"  period_size = %lu frames (<).", frames);
 			    break;
 			  case 0:
-			    debug(1,"  period_size = %lu frames (precisely).", frames);
+			    debug(log_level,"  period_size = %lu frames (precisely).", frames);
 			    break;
 			  case 1:
-			    debug(1,"  period_size = %lu frames (>).", frames);
+			    debug(log_level,"  period_size = %lu frames (>).", frames);
 			    break;
 			}
 
 			snd_pcm_hw_params_get_buffer_time(alsa_params,&uval, &dir);
 			switch (dir) {
 			  case -1:
-			    debug(1,"  buffer_time = %u us (<).", uval);
+			    debug(log_level,"  buffer_time = %u us (<).", uval);
 			    break;
 			  case 0:
-			    debug(1,"  buffer_time = %u us (precisely).", uval);
+			    debug(log_level,"  buffer_time = %u us (precisely).", uval);
 			    break;
 			  case 1:
-			    debug(1,"  buffer_time = %u us (>).", uval);
+			    debug(log_level,"  buffer_time = %u us (>).", uval);
 			    break;
 			}
 
 			snd_pcm_hw_params_get_buffer_size(alsa_params,&frames);
 			switch (dir) {
 			  case -1:
-			    debug(1,"  buffer_size = %lu frames (<).", frames);
+			    debug(log_level,"  buffer_size = %lu frames (<).", frames);
 			    break;
 			  case 0:
-			    debug(1,"  buffer_size = %lu frames (precisely).", frames);
+			    debug(log_level,"  buffer_size = %lu frames (precisely).", frames);
 			    break;
 			  case 1:
-			    debug(1,"  buffer_size = %lu frames (>).", frames);
+			    debug(log_level,"  buffer_size = %lu frames (>).", frames);
 			    break;
 			}
 			
 			snd_pcm_hw_params_get_periods(alsa_params, &uval, &dir);
 			switch (dir) {
 			  case -1:
-			    debug(1,"  periods_per_buffer = %u (<).", uval);
+			    debug(log_level,"  periods_per_buffer = %u (<).", uval);
 			    break;
 			  case 0:
-			    debug(1,"  periods_per_buffer = %u (precisely).", uval);
+			    debug(log_level,"  periods_per_buffer = %u (precisely).", uval);
 			    break;
 			  case 1:
-			    debug(1,"  periods_per_buffer = %u (>).", uval);
+			    debug(log_level,"  periods_per_buffer = %u (>).", uval);
 			    break;
 			}
 		}
@@ -645,9 +667,10 @@ static void play(short buf[], int samples) {
   if (alsa_handle == NULL) {
     pthread_mutex_lock(&alsa_mutex);
     ret = open_alsa_device();
-    open_mixer();
+    if (hardware_mixer)
+    	open_mixer();
     pthread_mutex_unlock(&alsa_mutex);
-    if ((ret == 0) && (audio_alsa.volume))
+    if ((hardware_mixer) && (ret == 0) && (audio_alsa.volume))
       audio_alsa.volume(set_volume);
   }
   if (ret == 0) {
@@ -656,11 +679,17 @@ static void play(short buf[], int samples) {
     int err, ignore;
     if ((snd_pcm_state(alsa_handle) == SND_PCM_STATE_PREPARED) ||
         (snd_pcm_state(alsa_handle) == SND_PCM_STATE_RUNNING)) {
-      err = snd_pcm_writei(alsa_handle, (char *)buf, samples);
-      if (err < 0) {
-        debug(1, "Error %d writing %d samples in play(): \"%s\".", err, samples,
-              snd_strerror(err));
-        ignore = snd_pcm_recover(alsa_handle, err, 1);
+      if (buf==NULL)
+      	debug(1,"NULL buffer passed to pcm_writei -- skipping it");
+      if (samples==0)
+      	debug(1,"empty buffer being passed to pcm_writei -- skipping it");
+      if ((samples!=0) && (buf!=NULL)) {
+				err = snd_pcm_writei(alsa_handle, (char *)buf, samples);
+				if (err < 0) {
+					debug(1, "Error %d writing %d samples in play(): \"%s\".", err, samples,
+								snd_strerror(err));
+					ignore = snd_pcm_recover(alsa_handle, err, 1);
+				}
       }
     } else {
       debug(1, "Error -- ALSA device in incorrect state (%d) for play.",
@@ -728,11 +757,30 @@ static void volume(double vol) {
 	debug(2, "Setting volume db to %f.", vol);
   set_volume = vol;
   if (hardware_mixer && alsa_mix_handle) {
-    if (snd_mixer_selem_set_playback_dB_all(alsa_mix_elem, vol, 0) != 0) {
-      debug(1, "Can't set playback volume accurately to %f dB.", vol);
-      if (snd_mixer_selem_set_playback_dB_all(alsa_mix_elem, vol, -1) != 0)
-        if (snd_mixer_selem_set_playback_dB_all(alsa_mix_elem, vol, 1) != 0)
-          die("Failed to set playback dB volume");
+    if (has_softvol) {
+      if (ctl && elem_id) {
+        snd_ctl_elem_value_t *value;
+        long raw;
+
+        if (snd_ctl_convert_from_dB(ctl, elem_id, (long) vol, &raw, 0) < 0)
+        debug(1, "Failed converting dB gain to raw volume value for the "
+                 "software volume control.");
+
+        snd_ctl_elem_value_alloca(&value);
+        snd_ctl_elem_value_set_id(value, elem_id);
+        snd_ctl_elem_value_set_integer(value, 0, raw);
+        snd_ctl_elem_value_set_integer(value, 1, raw);
+        if (snd_ctl_elem_write(ctl, value) < 0)
+        debug(1, "Failed to set playback dB volume for the software volume "
+                 "control.");
+      }
+    } else {
+      if (snd_mixer_selem_set_playback_dB_all(alsa_mix_elem, vol, 0) != 0) {
+        debug(1, "Can't set playback volume accurately to %f dB.", vol);
+        if (snd_mixer_selem_set_playback_dB_all(alsa_mix_elem, vol, -1) != 0)
+          if (snd_mixer_selem_set_playback_dB_all(alsa_mix_elem, vol, 1) != 0)
+            die("Failed to set playback dB volume");
+      }
     }
   }
   pthread_mutex_unlock(&alsa_mutex);
