@@ -437,7 +437,7 @@ double vol2attn(double vol, long max_db, long min_db) {
 
 #define order 3
 
-  double vol_setting = max_db;
+  double vol_setting = 0;
 
   if ((vol <= 0.0) && (vol >= -30.0)) {
     long range_db = max_db - min_db; // this will be a positive nunmber
@@ -465,6 +465,7 @@ double vol2attn(double vol, long max_db, long min_db) {
   } else if (vol != -144.0) {
     debug(1, "Volume request value %f is out of range: should be from 0.0 to -30.0 or -144.0.",
           vol);
+    vol_setting = min_db; // for safety, return the lowest setting...
   } else {
     vol_setting = min_db; // for safety, return the lowest setting...
   }
@@ -512,27 +513,68 @@ uint64_t get_absolute_time_in_fp() {
   return time_now_fp;
 }
 
+#ifdef CONFIG_METADATA
 ssize_t non_blocking_write(int fd, const void *buf, size_t count) {
-  // debug(1,"writing %u to pipe...",count);
-  // we are assuming that the count is always smaller than the FIFO's buffer
+	void *ibuf = (void *)buf;
+	size_t bytes_remaining = count;
+	int rc = 0;
   struct pollfd ufds[1];
-  ssize_t reply;
-  do {
-    ufds[0].fd = fd;
+	while ((bytes_remaining>0) && (rc==0)) {
+		// check that we can do some writing
+		ufds[0].fd = fd;
     ufds[0].events = POLLOUT;
-    int rv = poll(ufds, 1, 5000);
-    if (rv == -1)
-      debug(1, "error waiting for pipe to unblock...");
-    if (rv == 0)
-      debug(1, "timeout waiting for pipe to unblock");
-    reply = write(fd, buf, count);
-    if ((reply == -1) && ((errno == EAGAIN) || (errno == EWOULDBLOCK)))
-      debug(1, "writing to pipe will block...");
-    //    else
-    //      debug(1,"writing %u to pipe done...",reply);
-  } while ((reply == -1) && ((errno == EAGAIN) || (errno == EWOULDBLOCK)));
-  return reply;
-
+    rc = poll(ufds, 1, config.metadata_pipe_timeout);
+    if (rc < 0) {
+      // debug(1, "non-blocking write error waiting for pipe to become ready for writing...");
+    } else if (rc == 0) {
+      // warn("non-blocking write timeout waiting for pipe to become ready for writing");
+      rc = -2;
+    } else { //rc > 0, implying it might be ready
+    	size_t bytes_written = write(fd,ibuf,bytes_remaining);
+    	if (bytes_written==-1) {
+    	  // debug(1,"Error %d in non_blocking_write: \"%s\".",errno,strerror(errno));
+    		rc = -1;
+    	} else {
+    		ibuf += bytes_written;
+    		bytes_remaining -= bytes_written;
+    	}    		
+    }
+	}
+	if (rc==0)
+		return count-bytes_remaining; // this is just to mimic a normal write/3.
+	else
+		return rc;
   //  return write(fd,buf,count);
 }
+#endif
 
+/* from http://coding.debuntu.org/c-implementing-str_replace-replace-all-occurrences-substring#comment-722 */
+
+char *str_replace ( const char *string, const char *substr, const char *replacement ){
+  char *tok = NULL;
+  char *newstr = NULL;
+  char *oldstr = NULL;
+  char *head = NULL;
+ 
+  /* if either substr or replacement is NULL, duplicate string a let caller handle it */
+  if ( substr == NULL || replacement == NULL ) return strdup (string);
+  newstr = strdup (string);
+  head = newstr;
+  while ( (tok = strstr ( head, substr ))){
+    oldstr = newstr;
+    newstr = malloc ( strlen ( oldstr ) - strlen ( substr ) + strlen ( replacement ) + 1 );
+    /*failed to alloc mem, free old string and return NULL */
+    if ( newstr == NULL ){
+      free (oldstr);
+      return NULL;
+    }
+    memcpy ( newstr, oldstr, tok - oldstr );
+    memcpy ( newstr + (tok - oldstr), replacement, strlen ( replacement ) );
+    memcpy ( newstr + (tok - oldstr) + strlen( replacement ), tok + strlen ( substr ), strlen ( oldstr ) - strlen ( substr ) - ( tok - oldstr ) );
+    memset ( newstr + strlen ( oldstr ) - strlen ( substr ) + strlen ( replacement ) , 0, 1 );
+    /* move back head right after the last replacement */
+    head = newstr + (tok - oldstr) + strlen( replacement );
+    free (oldstr);
+  }
+  return newstr;
+}
