@@ -59,7 +59,9 @@ typedef struct time_ping_record {
 static int running = 0;
 
 static char client_ip_string[INET6_ADDRSTRLEN]; // the ip string pointing to the client
-static short client_ip_family;                  // AF_INET / AF_INET6
+static char self_ip_string[INET6_ADDRSTRLEN]; // the ip string being used by this program -- it could be one of many, so we need to know it
+static uint32_t self_scope_id; // if it's an ipv6 connection, this will be its scope
+static short connection_ip_family;                  // AF_INET / AF_INET6
 static uint32_t client_active_remote;           // used when you want to control the client...
 
 static SOCKADDR rtp_client_control_socket; // a socket pointing to the control port of the client
@@ -543,60 +545,48 @@ void *rtp_timing_receiver(void *arg) {
   return NULL;
 }
 
-static int bind_port(SOCKADDR *remote, int *sock) {
-  struct addrinfo hints, *info;
-
-  memset(&hints, 0, sizeof(hints));
-  hints.ai_family = remote->SAFAMILY;
-  hints.ai_socktype = SOCK_DGRAM;
-  hints.ai_flags = AI_PASSIVE;
-
-  char buffer[10];
-  
+static int bind_port(int ip_family,const char *self_ip_address,uint32_t scope_id,int *sock) { 
   // look for a port in the range, if any was specified.
   int desired_port = config.udp_port_base;
   int ret;
+  
+  int local_socket = socket(ip_family, SOCK_DGRAM, IPPROTO_UDP);
+  if (local_socket== -1)
+    die("Could not allocate a socket.");
+  SOCKADDR myaddr;
   do {
-    snprintf(buffer, 10, "%d", desired_port);
-
-    ret = getaddrinfo(NULL, buffer, &hints, &info);
-
-    if (ret < 0)
-      die("failed to get usable addrinfo?! %s.", gai_strerror(ret));
-
-    *sock = socket(remote->SAFAMILY, SOCK_DGRAM, IPPROTO_UDP);
-    
-/*
-  // this doesn't compile properly with OpenWrt Barrier Breaker.
-    #if defined(__linux__)
-    #ifdef AF_INET6
-    // now, if we are on IPv6, prefer a public ipv6 address
-    if (remote->SAFAMILY==AF_INET6) {
-      int value = IPV6_PREFER_SRC_PUBLIC;
-      ret = setsockopt(*sock, IPPROTO_IPV6, IPV6_ADDR_PREFERENCES, &value, sizeof(value));
-      if (ret<0)
-        die("error: could not select a preference for public IPv6 address");
+    memset(&myaddr,0,sizeof(myaddr));
+    if (ip_family==AF_INET) {
+      struct sockaddr_in *sa = (struct sockaddr_in *)&myaddr;
+      sa->sin_family = AF_INET;
+      sa->sin_port = ntohs(desired_port);
+      inet_pton(AF_INET,self_ip_address,&(sa->sin_addr));
+      ret = bind(local_socket,(struct sockaddr*)sa, sizeof(struct sockaddr_in));
     }
-    #endif
-    #endif
-*/
-    ret = bind(*sock, info->ai_addr, info->ai_addrlen);
-
-    freeaddrinfo(info);
-
+#ifdef AF_INET6
+    if (ip_family==AF_INET6) {
+      struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)&myaddr;
+      sa6->sin6_family = AF_INET6;
+      sa6->sin6_port = ntohs(desired_port);
+      inet_pton(AF_INET6,self_ip_address,&(sa6->sin6_addr));
+      sa6->sin6_scope_id=scope_id;
+      ret = bind(local_socket,(struct sockaddr*)sa6, sizeof(struct sockaddr_in6));
+    }
+#endif   
+    
   } while ((ret<0) && (errno==EADDRINUSE) && (desired_port!=0) && (desired_port++ < config.udp_port_base+config.udp_port_range));
   
   // debug(1,"UDP port chosen: %d.",desired_port);
   
   if (ret < 0) {
+     close(local_socket);
      die("error: could not bind a UDP port!");
   }
  
-  
   int sport;
   SOCKADDR local;
   socklen_t local_len = sizeof(local);
-  getsockname(*sock, (struct sockaddr *)&local, &local_len);
+  getsockname(local_socket, (struct sockaddr *)&local, &local_len);
 #ifdef AF_INET6
   if (local.SAFAMILY == AF_INET6) {
     struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)&local;
@@ -607,12 +597,18 @@ static int bind_port(SOCKADDR *remote, int *sock) {
     struct sockaddr_in *sa = (struct sockaddr_in *)&local;
     sport = ntohs(sa->sin_port);
   }
-
+  
+  *sock = local_socket;
   return sport;
 }
 
-void rtp_setup(SOCKADDR *remote, int cport, int tport, uint32_t active_remote, int *lsport,
+void rtp_setup(SOCKADDR *local, SOCKADDR *remote, int cport, int tport, uint32_t active_remote, int *lsport,
                int *lcport, int *ltport) {
+               
+  // this gets the local and remote ip numbers (and ports used for the TCD stuff)
+  // we use the local stuff to specify the address we are coming from and
+  // we use the remote stuff to specify where we're goint to
+  
   if (running)
     die("rtp_setup called with active stream!");
 
@@ -621,28 +617,40 @@ void rtp_setup(SOCKADDR *remote, int cport, int tport, uint32_t active_remote, i
   client_active_remote = active_remote;
 
   // print out what we know about the client
-  void *addr;
-  char *ipver;
-  int port;
-  char portstr[20];
-  client_ip_family = remote->SAFAMILY; // keep information about the kind of ip of the client
+  void *client_addr,*self_addr;
+  int client_port,self_port;
+  char client_port_str[64];
+  char self_addr_str[64];
+  
+  connection_ip_family = remote->SAFAMILY; // keep information about the kind of ip of the client
+  
 #ifdef AF_INET6
-  if (remote->SAFAMILY == AF_INET6) {
+  if (connection_ip_family == AF_INET6) {
     struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)remote;
-    addr = &(sa6->sin6_addr);
-    port = ntohs(sa6->sin6_port);
-    ipver = "IPv6";
+    client_addr = &(sa6->sin6_addr);
+    client_port = ntohs(sa6->sin6_port);
+    sa6 = (struct sockaddr_in6 *)local;
+    self_addr = &(sa6->sin6_addr);
+    self_port = ntohs(sa6->sin6_port);
+    self_scope_id = sa6->sin6_scope_id;
   }
 #endif
-  if (remote->SAFAMILY == AF_INET) {
+  if (connection_ip_family == AF_INET) {
     struct sockaddr_in *sa4 = (struct sockaddr_in *)remote;
-    addr = &(sa4->sin_addr);
-    port = ntohs(sa4->sin_port);
-    ipver = "IPv4";
+    client_addr = &(sa4->sin_addr);
+    client_port = ntohs(sa4->sin_port);
+    sa4 = (struct sockaddr_in *)local;
+    self_addr = &(sa4->sin_addr);
+    self_port = ntohs(sa4->sin_port);
   }
-  inet_ntop(remote->SAFAMILY, addr, client_ip_string,
-            sizeof(client_ip_string)); // keep the client's ip number
-  debug(1, "Connection from %s: %s:%d", ipver, client_ip_string, port);
+
+  inet_ntop(connection_ip_family, client_addr, client_ip_string,
+            sizeof(client_ip_string));
+  inet_ntop(connection_ip_family, self_addr, self_ip_string,
+            sizeof(self_ip_string));
+
+  debug(1, "Set up play connection from %s to self at %s.", client_ip_string,self_ip_string);
+
 
   // set up a the record of the remote's control socket
   struct addrinfo hints;
@@ -650,8 +658,9 @@ void rtp_setup(SOCKADDR *remote, int cport, int tport, uint32_t active_remote, i
 
   memset(&rtp_client_control_socket, 0, sizeof(rtp_client_control_socket));
   memset(&hints, 0, sizeof hints);
-  hints.ai_family = remote->SAFAMILY;
+  hints.ai_family = connection_ip_family;
   hints.ai_socktype = SOCK_DGRAM;
+  char portstr[20];
   snprintf(portstr, 20, "%d", cport);
   if (getaddrinfo(client_ip_string, portstr, &hints, &servinfo) != 0)
     die("Can't get address of client's control port");
@@ -667,7 +676,7 @@ void rtp_setup(SOCKADDR *remote, int cport, int tport, uint32_t active_remote, i
   // set up a the record of the remote's timing socket
   memset(&rtp_client_timing_socket, 0, sizeof(rtp_client_timing_socket));
   memset(&hints, 0, sizeof hints);
-  hints.ai_family = remote->SAFAMILY;
+  hints.ai_family = connection_ip_family;
   hints.ai_socktype = SOCK_DGRAM;
   snprintf(portstr, 20, "%d", tport);
   if (getaddrinfo(client_ip_string, portstr, &hints, &servinfo) != 0)
@@ -683,9 +692,9 @@ void rtp_setup(SOCKADDR *remote, int cport, int tport, uint32_t active_remote, i
   // now, we open three sockets -- one for the audio stream, one for the timing and one for the
   // control
 
-  *lsport = bind_port(remote, &audio_socket);
-  *lcport = bind_port(remote, &control_socket);
-  *ltport = bind_port(remote, &timing_socket);
+  *lsport = bind_port(connection_ip_family,self_ip_string,self_scope_id,&audio_socket);
+  *lcport = bind_port(connection_ip_family,self_ip_string,self_scope_id,&control_socket);
+  *ltport = bind_port(connection_ip_family,self_ip_string,self_scope_id,&timing_socket);
 
   debug(2, "listening for audio, control and timing on ports %d, %d, %d.", *lsport, *lcport,
         *ltport);
